import datetime
import pytz
import uuid
from django.db import models
from django.core.validators import MaxValueValidator
from django.utils.translation import gettext_lazy as _
from tinymce.models import HTMLField


class TimeStampBase(models.Model):
    date_created = models.DateTimeField(auto_now_add=True, null=True)
    date_modified = models.DateTimeField(auto_now=True, null=True)

    class Meta:
        abstract = True

class HazardLevels(models.Model):
    title = models.CharField(max_length=32)
    danger_level = models.IntegerField(validators=[MaxValueValidator(10)])
    color_code = models.CharField(max_length=6, null=True)
    description = models.TextField()

    class Meta:
        verbose_name = _('Hazard Level')
        verbose_name_plural = _('Hazard Levels')

class HazardFeeds(TimeStampBase):
    id = models.BigIntegerField(primary_key=True)
    date = models.DateTimeField()
    title = models.CharField(max_length=128)
    link = models.URLField()
    summary = models.TextField()
    hazard_level = models.ForeignKey(HazardLevels, on_delete=models.CASCADE)
    is_sent = models.BooleanField(default=False)
    date_send = models.DateTimeField(null=True)

    class Meta:
        verbose_name = _('Hazard Feed')
        verbose_name_plural = _('Hazard Feeds')

    def resency(self):
        now = datetime.datetime.utcnow().replace(tzinfo=pytz.utc)
        return now-self.date

    def is_newer_that(self, timedelta):
        return self.date_created-self.date < timedelta

    def date_send_set(self):
        self.date_send = datetime.datetime.utcnow().replace(tzinfo=pytz.utc)

    @classmethod
    def not_sent(cls):
        return cls.objects.filter(is_sent=False)



class WeatherRecipients(models.Model):
    email = models.EmailField()
    title = models.CharField(max_length=64, null=True)
    uuid = models.UUIDField(default=uuid.uuid4, editable=False, unique=True)
    is_active = models.BooleanField(default=False)

    class Meta:
        verbose_name = _('Weather hazard recipient')
        verbose_name_plural = _('Weather hazard recipients')

class EmailTemplates(models.Model):
    title = models.CharField(max_length=64, editable=False, unique=True)
    template = HTMLField(null=True)

    class Meta:
        verbose_name = _('Email Template')
        verbose_name_plural = _('Email Templates')



from django.core.management.base import BaseCommand
import django_rq
from rq import Queue
from redis import Redis
import datetime
from rq_scheduler import Scheduler

redis_conn = Redis('default')
q = Queue(connection=redis_conn)
# queue = django_rq.get_queue('default')
scheduler = Scheduler(queue=q)

def clear_scheduled_jobs():
    # Delete any existing jobs in the scheduler when the app starts up
    for job in scheduler.get_jobs():
        job.delete()

def register_scheduled_jobs():
    # do your scheduling here
    scheduler.schedule(scheduled_time=datetime.datetime.utcnow() + datetime.timedelta(minutes=2),
                           func='hazard_feed.jobs.parse_feeds',
                           interval=60*20)

    class Command(BaseCommand):

        def handle(self, *args, **options):
            clear_scheduled_jobs()
            register_scheduled_jobs()


import datetime
import django_rq
from .jobs import send_notification
from redis.exceptions import ConnectionError

def send_hazard_feed_notification(sender, instance, created, **kwargs):
    if created \
            and not instance.is_sent and (
                instance.date.date() == instance.date_created.date()
                or instance.is_newer_that(datetime.timedelta(hours=1))
            ):
        try:
            queue = django_rq.get_queue()
            queue.enqueue(send_notification, instance)
            instance.date_send_set()
            instance.is_sent = True
            instance.save()
        except ConnectionError:
            pass



# Generated by Django 3.0.2 on 2020-02-06 04:40

from django.db import migrations, models
import tinymce.models


class Migration(migrations.Migration):

    dependencies = [
        ('hazard_feed', '0006_weatherrecipients_uuid'),
    ]

    operations = [
        migrations.CreateModel(
            name='EmailTemplates',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(editable=False, max_length=64, unique=True)),
                ('template', tinymce.models.HTMLField(null=True)),
            ],
            options={
                'verbose_name': 'Email Template',
                'verbose_name_plural': 'Email Templates',
            },
        ),
    ]



from django_rq import job
from .config import WEATHER_FEED_URL
import asyncio
from hazard_feed.utils import (
    parse_weather_feeds, put_feed_to_db,
    make_weather_hazard_message, send_weather_mail,
    get_weather_recipients
    )

@job
def parse_feeds():
    feeds = parse_weather_feeds(WEATHER_FEED_URL)
    for feed in feeds:
        put_feed_to_db(feed)


@job
def send_notification(feed):
    recipients = get_weather_recipients()
    msg = make_weather_hazard_message(feed)
    event_loop = asyncio.get_event_loop()
    event_loop.run_until_complete(send_weather_mail(msg, recipients))


from django.test import TestCase, override_settings
from .utils import *
from .config import WEATHER_FEED_URL
from django.apps import apps
import asyncio
import django_rq
from rq_scheduler import Scheduler
from .jobs import parse_feeds
from .models import WeatherRecipients
import time


class TestHazardFeeds(TestCase):
    fixtures = ['hazard_feed/fixtures/hazard_levels.json']

    def setUp(self):
        pass

    def test_put_feed_to_db(self):
        feeds = parse_weather_feeds(WEATHER_FEED_URL)
        self.assertFalse(put_feed_to_db(feeds[0]))
        if len(feeds) > 1:
            self.assertTrue(put_feed_to_db(feeds[1]))

    def test_date_compare(self):
        d1 = datetime.datetime.utcnow()
        d2 = datetime.datetime.utcnow()+datetime.timedelta(hours=3)
        d3 = datetime.datetime.utcnow()+datetime.timedelta(days=1)
        # self.assertEqual(d1.date(), d2.date())
        self.assertNotEqual(d1.date(), d3.date())


    def test_send_weather_mail(self):
        feeds = parse_weather_feeds(WEATHER_FEED_URL)
        msg = make_weather_hazard_message(feeds[0])
        recipients = get_weather_recipients()
        event_loop = asyncio.get_event_loop()
        event_loop.run_until_complete(send_weather_mail(msg, recipients))

    def test_rq(self):
        # queue = django_rq.get_queue('default')
        # queue.enqueue(parse_feeds)
        redis_conn = django_rq.get_connection
        scheduler = Scheduler(connection=redis_conn)
        scheduler.schedule(scheduled_time=datetime.datetime.utcnow()+datetime.timedelta(seconds=5),
                               func=parse_feeds,
                               interval=60
                               )
    def test_notify_signal(self):
        feed = HazardFeeds.objects.create(
            id=1580800025,
            date=datetime.datetime.utcnow(),
            date_modified=datetime.datetime.utcnow()+datetime.timedelta(minutes=5),
            title='Предупреждение о неблагоприятном явлении',
            link='http://www.pogoda.by/news/?page=34647',
            summary='Желтый уровень опасности. 5 февраля (среда) на '
                    'отдельных участках дорог республики ожидается гололедица.',
            hazard_level=HazardLevels.objects.get(id=3),
            is_sent=False
        )
        self.assertTrue(feed.is_sent)
        print(feed.date_created)
        print(feed.date_modified)

    def test_recipient(self):
        self.assertIsInstance(get_weather_recipients(), list)

    def test_templated_msg(self):
        feed = HazardFeeds(
            id=1580800025,
            date=datetime.datetime.utcnow(),
            date_modified=datetime.datetime.utcnow() + datetime.timedelta(minutes=5),
            title='Предупреждение о неблагоприятном явлении',
            link='http://www.pogoda.by/news/?page=34647',
            summary='Желтый уровень опасности. 5 февраля (среда) на '
                    'отдельных участках дорог республики ожидается гололедица.',
            hazard_level=HazardLevels.objects.get(id=3),
            is_sent=False
        )
        msg = make_weather_hazard_message(feed)
        print(msg.get_payload()[0])



from django.contrib import admin
from .models import (
    HazardLevels, HazardFeeds,
    WeatherRecipients, EmailTemplates
)

@admin.register(HazardLevels)
class HazardLevelsAdmin(admin.ModelAdmin):
    pass

@admin.register(HazardFeeds)
class HazardFeedsAdmin(admin.ModelAdmin):
    pass

@admin.register(WeatherRecipients)
class WeatherRecipientsAdmin(admin.ModelAdmin):
    pass

@admin.register(EmailTemplates)
class EmailTemplateAdmin(admin.ModelAdmin):
    readonly_fields = (['title'])




import setuptools
try: # for pip >= 10
    from pip._internal.req import parse_requirements
except ImportError: # for pip <= 9.0.3
    from pip.req import parse_requirements


with open("README.md", "r") as fh:
    long_description = fh.read()

install_reqs = parse_requirements('./requirements.txt', session='hack')

reqs = [str(ir.req) for ir in install_reqs]

setuptools.setup(
    name="hazard-feed",
    version="0.1.3.1",
    author="Aleksandr Nikitin",
    author_email="hitnik@gmail.com",
    description="pogoda.by storm warning rss parser",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/hitnik/hazard_feed",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
    install_requires=reqs,
)


# Generated by Django 3.0.2 on 2020-01-26 21:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('hazard_feed', '0002_hazardlevels_color_code'),
    ]

    operations = [
        migrations.CreateModel(
            name='WeatherRecipients',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('email', models.EmailField(max_length=254)),
                ('title', models.CharField(max_length=64, null=True)),
                ('is_active', models.BooleanField(default=False)),
            ],
            options={
                'verbose_name': 'Weather hazard recipient',
                'verbose_name_plural': 'Weather hazard recipients',
            },
        ),
    ]



# Generated by Django 3.0.2 on 2020-02-04 20:28

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('hazard_feed', '0005_auto_20200204_2313'),
    ]

    operations = [
        migrations.AddField(
            model_name='weatherrecipients',
            name='uuid',
            field=models.UUIDField(default=uuid.uuid4, editable=False, unique=True),
        ),
    ]



import re
import feedparser
import time
import datetime
import pytz
from .models import (
    HazardLevels, HazardFeeds,
    WeatherRecipients, EmailTemplates
)
from django.conf import settings
import aiosmtplib
from django.template import loader
from email.message import EmailMessage
from bs4 import BeautifulSoup
from django.apps import apps
from django.template import Context, Template
from django.db.utils import OperationalError

def hazard_level_in_text_find(text):
    """
    chek if hazard level in text
    :param text:
    :return:
    """
    try:
        for hazard in HazardLevels.objects.all():
            if re.search(hazard.title, text):
                return hazard
    except OperationalError:
        return None
    return None

def parse_weather_feeds(url):
    """
    parse weather hazard rss to django model
    :param url:url of weather page
    :return:
    """
    feeds = feedparser.parse(url)
    feeds_out = []
    for feed in feeds.entries:
        ms = int(time.mktime(feed.published_parsed))
        date = datetime.datetime.fromtimestamp(ms).replace(tzinfo=pytz.utc)
        # date_parsed = datetime.datetime.utcnow().replace(tzinfo=pytz.utc)
        hazard_level = hazard_level_in_text_find(feed.summary)
        if hazard_level:
            hazard_feed = HazardFeeds(
                id=feed.id, date=date, title=feed.title,
                link=feed.link, summary=feed.summary,
                hazard_level=hazard_level,
                is_sent=False
            )
            feeds_out.append(hazard_feed)
        else:
            raise Exception('Hazard level define error')
    return feeds_out

def put_feed_to_db(feed):
    try:
        if not HazardFeeds.objects.filter(id=feed.id).exists():
            feed.save()
            return True
    except OperationalError:
        return False
    return False

def make_weather_hazard_message(feed):
    date = datetime.datetime.utcnow().replace(tzinfo=pytz.utc)
    local_tz = pytz.timezone(settings.TIME_ZONE)
    date = date.astimezone(local_tz)
    template = Template(EmailTemplates.objects.get(title='weather_mail').template)
    context = Context({'date': date, 'feed': feed})
    html = template.render(context)
    soup = BeautifulSoup(html, 'html.parser')
    text = soup.get_text()
    msg = EmailMessage()
    msg['From'] = settings.WEATHER_EMAIL_FROM
    msg['Subject'] = feed.title
    msg.set_content(text)
    msg.add_alternative(html, subtype='html')
    return msg

def get_weather_recipients():
    return list(WeatherRecipients.objects.filter(is_active=True).values_list('email', flat=True))



async def send_weather_mail(msg, recipients):

    """
    try to get queryset with async
    :param msg:
    :param recipients:
    :return:
    """
    config = apps.get_app_config('hazard_feed')
    if isinstance(recipients, list) and len(recipients) > 0:
        if config.WEATHER_USE_TSL:
            await aiosmtplib.send(
                msg,
                hostname=config.WEATHER_EMAIL_SMTP_HOST,
                port=config.WEATHER_EMAIL_SMTP_PORT,
                use_tls=config.WEATHER_USE_TSL,
                username=config.WEATHER_EMAIL_HOST_USER,
                password=config.WEATHER_EMAIL_HOST_PASSWORD,
                sender=settings.WEATHER_EMAIL_FROM,
                recipients=recipients
            )
        else:
            await aiosmtplib.send(
                msg,
                hostname=config.WEATHER_EMAIL_SMTP_HOST,
                port=config.WEATHER_EMAIL_SMTP_PORT,
                username=config.WEATHER_EMAIL_HOST_USER,
                password=config.WEATHER_EMAIL_HOST_PASSWORD,
                sender=settings.WEATHER_EMAIL_FROM,
                recipients=recipients
            )

# def get_weather_mail():



# Generated by Django 3.0.2 on 2020-01-23 00:15

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='HazardLevels',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=32)),
                ('danger_level', models.IntegerField(validators=[django.core.validators.MaxValueValidator(10)])),
                ('description', models.TextField()),
            ],
            options={
                'verbose_name': 'Hazard Level',
                'verbose_name_plural': 'Hazard Levels',
            },
        ),
        migrations.CreateModel(
            name='HazardFeeds',
            fields=[
                ('id', models.BigIntegerField(primary_key=True, serialize=False)),
                ('date', models.DateTimeField()),
                ('title', models.CharField(max_length=128)),
                ('link', models.URLField()),
                ('summary', models.TextField()),
                ('date_modified', models.DateTimeField()),
                ('hazard_level', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='hazard_feed.HazardLevels')),
            ],
            options={
                'verbose_name': 'Hazard Feed',
                'verbose_name_plural': 'Hazard Feeds',
            },
        ),
    ]
